import turbogears
from turbogears import controllers, expose
from turbogears import flash, redirect, url
from pstnconfig import model
from paraxip.gateway_gui import extjs
from operator import itemgetter
import simplejson
import os

import logging
log = logging.getLogger("pstnconfig.controllers")

class CallControlController(controllers.Controller):
#################################################
#                                               #
# Channel Group methods (Add/delete/get/save)   #
#                                               #
#################################################
  ##########################################################################
  @expose()
  def addChannelGroup(self, **data):
    log.info("Adding Channel Group to config: %s" % data)
    channelData = simplejson.loads(data['channelGroup'])

    if channelData['direction'] == 'IN':
      model.addChannelGroup(model.currentConfig,
                            ID=channelData['id'],
                            direction=channelData['direction'])
    else:
      model.addChannelGroup(model.currentConfig,
                      ID=channelData['id'],
                      outboundHuntingScheme=channelData['outboundHuntingScheme'],
                      direction=channelData['direction'])

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deleteChannelGroup(self, **data):
    log.info("Deleting Channel Group: %s" % data)
    for selection in simplejson.loads(data['channelGroups']):
      channelGroup = model.getObjectFromEditorId(selection['nodeId'])
      model.currentConfig.get_callControl().get_channelGroups().get_channelGroup().remove(channelGroup)
      model.deleteObjectFromEditor(channelGroup)

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns the data for the interface associated to the given id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The wanpipeInterface number of this interface
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmChannelGroupData(self, id):
    log.info("Getting channel group data for channel id: %s" % id)
    # Get the instance of the interface
    channelGroup =  model.getObjectFromEditorId(id)
    data = {
      'id':channelGroup.get_ID(),
      'nodeId':id,
      'direction':channelGroup.get_direction(),
      'outboundHuntingScheme':channelGroup.get_outboundHuntingScheme()
    }             
    log.info("Loading Channel group data: %s" % data)
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Saves the new informations of the channel group.
  # All values are editable.
  @expose()
  def saveChannelGroup (self, **data):
    log.info("Saving channel group: %s" % data)
    channelData = simplejson.loads(data['channelGroup'])
    channelGroup = model.getObjectFromEditorId(channelData['nodeId'])

    if channelGroup:
      if channelData['direction'] == "IN":
        channelGroup.set_outboundHuntingScheme(None)
      model.saveAttributes(channelGroup,channelData)

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns a list of all the channel group configs available
  # in the system.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config.
  #   - The display name of the configs
  @expose() # Ext.form.Action.load - expects JSON
  def getGridChannelGroupData(self, id):
    # Get the physicalConfig for digital boards
    data = []
    for channelGroup in model.currentConfig.get_callControl().\
              get_channelGroups().get_channelGroup():
      data.append({
        'id': channelGroup.get_ID(),
        'direction': channelGroup.get_direction(), 
        'outboundHuntingScheme': channelGroup.get_outboundHuntingScheme(),
        'nodeId':model.getObjectId(channelGroup)
      })     
    data = sorted(data, key=itemgetter('id'))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

#################################################
#                                               #
# ISDN Config methods (Add/delete/get/save)     #
#                                               #
#################################################
  ##########################################################################
  @expose()
  def addIsdnConfig(self, **data):
    isdnData = simplejson.loads(data['isdnConfig'])

    if isdnData['isdnConfigChoice'] == 'FAS':
      # Default switch variant set to 5ESS for T1 and ETSI for E1.
      if model.containsT1Board(model.currentConfig):
        log.debug("Adding isdn config with T1, switch variant set to 5ESS")
        strSwitchVariant = '5ESS'
      else:
        log.debug("Adding isdn config with E1, switch variant set to ETSI")
        strSwitchVariant = 'ETSI'
      model.addFasConfig(model.currentConfig,
          displayName='new_FAS_%s' % model.getNextISDNConfID(model.currentConfig),
          termination='TERMINAL',
          switchVariant=strSwitchVariant,
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='YES',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS',
          callerNameLocation='IN-SETUP-MSG')
    elif isdnData['isdnConfigChoice'] == 'NFAS':
      model.addNfasGroup(model.currentConfig,
          displayName='new_NFAS_%s' % model.getNextISDNConfID(model.currentConfig),
          termination='TERMINAL',
          switchVariant='5ESS',
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='YES',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS',
          callerNameLocation='IN-SETUP-MSG')

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deleteIsdnConfig(self, **data):
    for selection in simplejson.loads(data['isdnConfigs']):
      isdnConfig = model.getObjectFromEditorId(selection['nodeId'])
      if type(isdnConfig) == model.fasConfig:
        model.currentConfig.get_callControl().get_isdn().get_fasConfig().remove(isdnConfig)
      elif type(isdnConfig) == model.nfasGroup:
        model.currentConfig.get_callControl().get_isdn().get_nfasGroup().remove(isdnConfig)
      model.deleteObjectFromEditor(isdnConfig)

    #If it was the last analog config, erase the <analog></analgo> from the config
    if len(model.currentConfig.get_callControl().get_isdn().get_fasConfig()) <= 0 and \
      len(model.currentConfig.get_callControl().get_isdn().get_nfasGroup()) <= 0:
      model.currentConfig.get_callControl().set_isdn(None)
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns a list of all the fas and nfasGroup configs available
  # in the system.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config.
  #   - The display name of the configs
  @expose() # Ext.form.Action.load - expects JSON
  def getGridISDNConfigurationData(self, id):
    data = []
    isdn = model.currentConfig.get_callControl().get_isdn()
    if isdn:
      for fas in (isdn.get_fasConfig() + isdn.get_nfasGroup()):
        data.append({
          'id':fas.get_ID(),
          'displayName':fas.get_displayName(),
          'nodeId':model.getObjectId(fas)
        })
    data = sorted(data, key=itemgetter('id'))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the value of all the attributes of the fas config.
  # The fields to be exposed to the user are:
  #   - Real ID of the config.
  #   - The display name of the config
  #   - Termination
  #   - Switch Variant
  #   - B-Channel Negotiation
  #   - Tone Set Attributes
  #   - Initiate Restart Procedure
  #   - Inband Progress Tones Generation
  #   - Inband Progress Tones Indicator
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmIsdnFasData(self, id):
    fasConfig = model.getObjectFromEditorId(id)

    data = {
      'idFas_text': fasConfig.get_ID(),
      'nodeId':id,
      'nameFas_text': fasConfig.get_displayName(),
      'terminationFas_select': fasConfig.get_termination(),
      'switchVariantFas_select': fasConfig.get_switchVariant(),
      'BChannelNegotiationFas_select':fasConfig.get_BChannelNegotiation(),
      'initRestartProcFas_select':fasConfig.get_initiateRestartProcedure(),
      'inbandProgTonesGenFas_select':fasConfig.get_inbandProgressTonesGeneration(),
      'inbandProgTonesIndFas_select':fasConfig.get_inbandProgressTonesIndicator(),
      'callerNameLocationFas_select':fasConfig.get_callerNameLocation(),
      'waitFacilityDelayMsFas_numfield':fasConfig.get_waitFacilityDelayMs()
    }                                  

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))
    
  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveFasConfig (self, **data):
    log.info("Saving fas config: %s" % data)
    fasData = simplejson.loads(data['fasConfig'])['fas']
    log.info("Decoded data: %s" % fasData)
    fasConfig = model.getObjectFromEditorId(fasData['nodeId'])

    if fasConfig:
      if fasData['callerNameLocation'] != "IN-FACILITY-MS":
        fasConfig.set_waitFacilityDelayMs(None)
      model.saveAttributes(fasConfig,fasData)

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns the value of all the attributes of the nfas config.
  # The fields to be exposed to the user are:
  #   - Real ID of the config.
  #   - The display name of the config
  #   - Termination
  #   - Switch Variant
  #   - B-Channel Negotiation
  #   - Tone Set Attributes
  #   - Initiate Restart Procedure
  #   - Inband Progress Tones Generation
  #   - Inband Progress Tones Indicator
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmIsdnNfasData(self, id):
    nfasGroup = model.getObjectFromEditorId(id)
    data = {
      'idNfas_text': nfasGroup.get_ID(),
      'nodeId':id,
      'nameNfas_text': nfasGroup.get_displayName(),
      'terminationNfas_select': nfasGroup.get_termination(),
      'switchVariantNfas_select': nfasGroup.get_switchVariant(),
      'BChannelNegotiationNfas_select':nfasGroup.get_BChannelNegotiation(),
      'initRestartProcNfas_select':nfasGroup.get_initiateRestartProcedure(),
      'inbandProgTonesGenNfas_select':nfasGroup.get_inbandProgressTonesGeneration(),
      'inbandProgTonesIndNfas_select':nfasGroup.get_inbandProgressTonesIndicator(),
      'callerNameLocationNfas_select':nfasGroup.get_callerNameLocation(),
      'waitFacilityDelayMsNfas_numfield':nfasGroup.get_waitFacilityDelayMs()
    }                                  

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveNfasConfig (self, **data):
    log.info("Saving nfas group config: %s" % data)
    nfasData = simplejson.loads(data['nfasGroup'])['nfas']
    nfasConfig = model.getObjectFromEditorId(nfasData['nodeId'])

    if nfasConfig:
      if nfasData['callerNameLocation'] != "IN-FACILITY-MS":
        nfasConfig.set_waitFacilityDelayMs(None)
      model.saveAttributes(nfasConfig,nfasData)

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

#################################################
#                                               #
# Analog Config methods (Add/delete/get/save)   #
#                                               #
#################################################
  ##########################################################################
  @expose()
  def addAnalogConfig(self, **data): 
    analogData = simplejson.loads(data['analogConfig'])

    if analogData['analogConfigChoice'] == 'FXO':
      configParams = {}
      configParams['displayName'] = 'new_FXO_%s' % model.getNextAnalogConfID(model.currentConfig)
      # TODO FINISH THIS
      model.addAnalogFxoConfig(model.currentConfig, configParams)
    # TODO => FXS
    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionSubmitResponse(success=True)))
  ##########################################################################
  @expose()
  def deleteAnalogConfig(self, **data):

    log.info("Deleting analog config %s" % data['analogConfigs'])

    for selection in simplejson.loads(data['analogConfigs']):
      analogConfig = model.getObjectFromEditorId(selection['nodeId'])
      if type(analogConfig) == model.analogFxoConfig:
        model.currentConfig.get_callControl().get_analog().get_analogFxoConfig().remove(analogConfig)
      elif type(analogConfig) == model.get_analogFxsConfig:
        model.currentConfig.get_callControl().get_analog().get_analogFxsConfig().remove(analogConfig)
      model.deleteObjectFromEditor(analogConfig)

    #If it was the last analog config, erase the <analog></analgo> from the config
    if not len(model.currentConfig.get_callControl().get_analog().get_analogFxoConfig()):
      model.currentConfig.get_callControl().set_analog(None)

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns a list of all the analogFxo and analogFxs configs available
  # in the system.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config.
  #   - The display name of the configs
  @expose() # Ext.form.Action.load - expects JSON
  def getGridAnalogConfigurationData(self, id):
    data = []
    analog = model.currentConfig.get_callControl().get_analog()
    if analog:
      for fxo in (analog.get_analogFxoConfig() + analog.get_analogFxsConfig()):
        data.append({
          'id':fxo.get_ID(),
          'displayName':fxo.get_displayName(),
          'nodeId':model.getObjectId(fxo)
        })
    data = sorted(data, key=itemgetter('id'))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the value of all the attributes of the analog fxo config.
  # The fields to be exposed to the user are:
  #   - Real ID of the config.
  #   - The display name of the config
  #   - Tone Set Name
  #   - Inbound Call Handling
  #   - Outbound Call Handling
  #   - Connect Supervision
  #   - Disconnect Supervision
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmAnalogFxoData(self, id):
    analogFxo = model.getObjectFromEditorId(id)
    notify =  analogFxo.get_inboundCallHandling().get_notifyOutboundSide()
    inbound = analogFxo.get_inboundCallHandling()
    out_dial = analogFxo.get_outboundCallHandling().get_dial()[0]
    con_super = analogFxo.get_outboundCallHandling().\
                get_connectSupervision()[0]
    ring_cadence = analogFxo.get_disconnectSupervision().\
                   get_disconnectDetectors().get_ringCadenceDetector()
    
    data = {
      'idFxo_text':analogFxo.get_ID(),
      'nodeId':id,
      'nameFxo_text':analogFxo.get_displayName(),
      'minimumOnHookDurationMs':analogFxo.get_minimumOnHookDurationMs(),
      'notifyOutbound_condition_select':notify.get_condition(),
      'numOfRings_text':notify.get_numberOfRings(),
      'notifyOutbound_delay_text':notify.get_delayInMs(),

      'notifyOutboundConnect_condition_select': \
        inbound.get_connect().get_condition(),
      'callerID_type_selection':inbound.get_callerID().get_type(),
      
      'dial_active_selection':out_dial.get_active(),
      'dial_waitForDialTone_selection': out_dial.get_waitForDialTone(),
      'preDialDelayInMs_text': out_dial.get_preDialDelayInMs(),
      'interDigitDelayInMs_text': out_dial.get_interDigitDelayInMs(),
      'dTMFDigitDurationInMs_text': out_dial.get_DTMFDigitDurationInMs(),

      'connectOutBoundSide_condition_selection': \
        con_super.get_connectOutboundSide().get_condition(),
      'connectParamOutbound_condition_selection': \
        con_super.get_connect().get_condition(),
      'connectParamOutbound_delayInMs_text': \
        con_super.get_connect().get_delayInMs(),

      'ringCadenceDetector_active_selection': \
        ring_cadence.get_active(),
      'ringCadenceDetector_OffDelayMs_text': \
        ring_cadence.get_ringCadenceOffDelayMs()

    }                                  
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))
    
  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveFxoConfig (self, **data):
    log.info("Saving fxo group config: %s" % data)
    fxoData = simplejson.loads(data['frmAnalogFxo'])
    log.info("Saving fxo Data config: %s" % fxoData)
    fxoConfig = model.getObjectFromEditorId(data['nodeId'])

    notify =  fxoConfig.get_inboundCallHandling().get_notifyOutboundSide()
    inbound = fxoConfig.get_inboundCallHandling()
    out_dial = fxoConfig.get_outboundCallHandling().get_dial()[0]
    con_super = fxoConfig.get_outboundCallHandling().\
                get_connectSupervision()[0]
    ring_cadence = fxoConfig.get_disconnectSupervision().\
                   get_disconnectDetectors().get_ringCadenceDetector()
    

    # Sections are: config, analog, audio, EchoCancellation, etc.
    # Values are the values of that section wich is a dictionary
    for section, values in fxoData.items():
      if section == 'fxoConfig':
        fxoConfig.set_displayName(values['displayName'])
        fxoConfig.set_minimumOnHookDurationMs(values['minimumOnHookDurationMs'])
      elif section == 'inboundCall':
        notify.set_condition(values['notifyOutbound_condition'])
        if values['notifyOutbound_condition'] == 'AFTER-N-RING':
          notify.set_numberOfRings(values['numberOfRings'])
        else:
          notify.set_numberOfRings(None)
        notify.set_delayInMs(values['notifyOutbound_delay'])
        inbound.get_connect().set_condition(values['connect_condition'])
        inbound.get_callerID().set_type(values['callerID_type'])
      elif section == 'outboundCallDialing':
        out_dial.set_active(values['dial_active'])
        out_dial.set_waitForDialTone(values['dial_waitForDialTone'])
        out_dial.set_preDialDelayInMs(values['preDialDelayInMs'])
        out_dial.set_interDigitDelayInMs(values['interDigitDelayInMs'])
        out_dial.set_DTMFDigitDurationInMs(values['dTMFDigitDurationInMs'])
      elif section == 'outboundCallConnect':
        con_super.get_connectOutboundSide().\
            set_condition(values['connectOutBoundSide_condition'])
        con_super.get_connect().\
            set_condition(values['connectParamOutbound_condition'])
        con_super.get_connect().\
            set_delayInMs(values['connectParamOutbound_delayInMs'])
      elif section == 'disconnectSupervision':
        ring_cadence.set_active(values['ringCadenceDetector_active'])
        if values['ringCadenceDetector_active'] == "YES":
          ring_cadence.\
              set_ringCadenceOffDelayMs(values['ringCadenceDetector_OffDelayMs'])
        else:
          ring_cadence.\
              set_ringCadenceOffDelayMs(None)
      else:
        # Get config attribute method
        method = getattr(fxoConfig, "get_%s" % section)
        model.saveAttributes(method(),values)

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  # TODO
  # def getFrmFxsData

  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveFxsConfig (self, **data):

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

#################################################
#                                               #
# ApplyTo FasConfigs                            #
#                                               #
#################################################
  ##########################################################################
  # Returns all the digital boards.
  @expose()
  def getBoards(self,id):
    data = []
    for board in (model.currentConfig.get_boards().get_sangomaDigitalBoard() + \
      model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard()):
      data.append({
        'id': board.get_ID(),
        'displayName':board.get_displayName(),
        'nodeId': model.getObjectId(board)
      })
    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns all the digital interfaces of the board corresponding to the 
  # given id.
  @expose()
  def getInterfaces(self,id):
    data = []
    board = model.getBoardObject(model.currentConfig, int(id))

    for interface in board.get_interface():
      data.append({
        'id': interface.get_ID(),
        'displayName': interface.get_displayName()
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))


  ##########################################################################
  # Returns all the channel groups.
  @expose()
  def getChannelGroups(self,id):
    data = []
    for channelGroup in model.currentConfig.get_callControl().get_channelGroups().get_channelGroup():
      data.append({
        'id': channelGroup.get_ID()
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data in the fas config applyTo section.
  # Returned values are:
  #   - Board id
  #   - Interface id
  #   - First Channel
  #   - Last Channel
  #   - Channel Group ID
  @expose()
  def getFasBChannels(self, id):
    data = []
    fasConfig = model.getISDNConfig(model.currentConfig, int(id))

    for interfaceRef in model.getInterfaceRefsForConfig(fasConfig):
      for bChannelRange in interfaceRef.get_BChannelRange():
        data.append({
          'boardId':interfaceRef.get_boardID(),
          'interfaceId':interfaceRef.get_interfaceID(),
          'firstChannel':bChannelRange.get_firstChannel(),
          'lastChannel':bChannelRange.get_lastChannel(),
          'channelGroupID':bChannelRange.get_channelGroupID(),
          'nodeId': model.getObjectId(model.getBoardObject(model.currentConfig, interfaceRef.get_boardID()))
        })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Apply the BChannel to the given fas configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyBChannelToFasConfig(self,**data):
    log.info("Applying BChannels to fas config: %s" % data)
    id = simplejson.loads(data['fasConfigID'])
    gridData = simplejson.loads(data['gridData'])

    fasConfig = model.getISDNConfig(model.currentConfig, int(id))
    fasConfig.set_applyTo(None) #Clear the applyTo section

    for interfaceInfo in gridData:
      #Find the board object
      board = model.getBoardObject(model.currentConfig,int(interfaceInfo['boardId']))
      #Find the interface object
      interface = model.getInterfaceObject(model.currentConfig, board.get_ID(), int(interfaceInfo['interfaceId']))
      BChannelRange = model.BChannelRange.factory(firstChannel=interfaceInfo['firstChannel'], 
        lastChannel=interfaceInfo['lastChannel'], channelGroupID=interfaceInfo['channelGroupID'])
      model.applyConfigToInterface(fasConfig,board,interface,[BChannelRange])

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

#################################################
#                                               #
# ApplyTo Nfas Group                            #
#                                               #
#################################################
  ##########################################################################
  # Returns the data in the nfas group members section.
  # Returned values are:
  #   - Board id
  #   - Interface id
  #   - Nfas Interface Id
  #   - DChannel Mode
  #   - First Channel
  #   - Last Channel
  #   - Channel Group ID
  @expose()
  def getNfasBChannels(self, id):
    data = []
    nfasGroup = model.getISDNConfig(model.currentConfig, int(id))

    for interfaceRef in model.getInterfaceRefsForNfasGroup(nfasGroup):
      for bChannelRange in interfaceRef.get_BChannelRange():
        data.append({
          'boardId':interfaceRef.get_boardID(),
          'interfaceId':interfaceRef.get_interfaceID(),
          'nfasInterfaceID':interfaceRef.get_nfasInterfaceID(),
          'DChannelMode':interfaceRef.get_DChannelMode(),
          'firstChannel':bChannelRange.get_firstChannel(),
          'lastChannel':bChannelRange.get_lastChannel(),
          'channelGroupID':bChannelRange.get_channelGroupID(),
          'nodeId': model.getObjectId(model.getBoardObject(model.currentConfig, interfaceRef.get_boardID()))
        })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns all the digital boards.
  @expose()
  def applyBChannelToNfasConfig(self,**data):
    log.info("Applying BChannels to nfas config: %s" % data)
    id = simplejson.loads(data['nfasGroupID'])
    gridData = simplejson.loads(data['gridData'])

    nfasGroup = model.getISDNConfig(model.currentConfig, int(id))
    nfasGroup.set_members(None) #Clear the members section

    for interfaceInfo in gridData:
      #Find the board object
      board = model.getBoardObject(model.currentConfig,int(interfaceInfo['boardId']))
      #Find the interface object
      interface = model.getInterfaceObject(model.currentConfig, board.get_ID(), int(interfaceInfo['interfaceId']))
      BChannelRange = model.BChannelRange.factory(firstChannel=interfaceInfo['firstChannel'], 
        lastChannel=interfaceInfo['lastChannel'], channelGroupID=interfaceInfo['channelGroupID'])
      model.addInterfaceToNfasMembers(nfasGroup,board,interface,
          interfaceInfo['nfasInterfaceID'],interfaceInfo['DChannelMode'],[BChannelRange])

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

#################################################
#                                               #
# ApplyTo AnalogFxoConfig                       #
#                                               #
#################################################
  ##########################################################################
  # Returns all the analog boards.
  @expose()
  def getAnalogBoards(self, id):
    data = []
    for board in model.currentConfig.get_boards().get_sangomaAnalogBoard():
      data.append({
        'id': board.get_ID(),
        'displayName':board.get_displayName(),
        'nodeId': model.getObjectId(board)
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns all the analog interfaces of the board corresponding to the 
  # given id.
  @expose()
  def getAnalogInterfaces(self, id):
    data = []
    board = model.getBoardObject(model.currentConfig, int(id))

    for interface in board.get_interface():
      data.append({
        'id': interface.get_ID(),
        'displayName': interface.get_displayName()
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data in the fxo config applyTo section.
  # Returned values are:
  #   - Board id
  #   - Interface id
  #   - Channel Group ID
  @expose()
  def getFxoInterfaces(self, id):
    data = []
    fxoConfig = model.getAnalogConfig(model.currentConfig, int(id))

    for interfaceRef in model.getInterfaceRefsForConfig(fxoConfig):
      data.append({
        'boardId':interfaceRef.get_boardID(),
        'interfaceId':interfaceRef.get_interfaceID(),
        'channelGroupID':interfaceRef.get_channelGroupID(),
        'nodeId': model.getObjectId(model.getBoardObject(model.currentConfig, interfaceRef.get_boardID()))
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))
  
  ##########################################################################
  # Apply the Interfaces to the given fxo configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyInterfacesToFxoConfig(self, **data):
    log.info("Applying Interfaces to fxo config: %s" % data)
    id = simplejson.loads(data['fxoConfigID'])
    gridData = simplejson.loads(data['gridData'])

    fxoConfig = model.getAnalogConfig(model.currentConfig, int(id))
    fxoConfig.set_applyTo(None) #Clear the applyTo section

    for interfaceInfo in gridData:
      #Find the board object
      board = model.getBoardObject(model.currentConfig,int(interfaceInfo['boardId']))
      #Find the interface object
      interface = model.getInterfaceObject(model.currentConfig, board.get_ID(), int(interfaceInfo['interfaceId']))
      model.applyConfigToInterface(fxoConfig,board,interface,channelGroupID=interfaceInfo['channelGroupID'])

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

  ##########################################################################
  # Returns the data in the fxo detector section.
  # Returned values are:
  #   - name
  #   - sequence
  @expose()
  def getFxoDetectors(self, id):
    data = []
    fxoConfig = model.getAnalogConfig(model.currentConfig, int(id))
  
    for detectorRef in fxoConfig.get_disconnectSupervision().\
      get_disconnectDetectors().get_detector():
      data.append({
        'name':detectorRef.get_name(),
        'sequence':detectorRef.get_sequence()
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

 ##########################################################################
  # Apply the Detectors to the given fxo configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyDetectorsToFxoConfig(self, **data):
    log.info("Applying Detectors to fxo config: %s" % data)
    id = simplejson.loads(data['fxoConfigID'])
    gridData = simplejson.loads(data['gridData'])

    fxoConfig = model.getAnalogConfig(model.currentConfig, int(id))
    fxoConfig.get_disconnectSupervision().\
    get_disconnectDetectors().set_detector([]) #Clear the detector section

    for detectorInfo in gridData:
      d = model.detector(detectorInfo['name'],detectorInfo['sequence'])
      fxoConfig.get_disconnectSupervision().get_disconnectDetectors().\
          add_detector(d)

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

