import logging

import turbogears
import os
import simplejson
import netborderPyThread
import urllib
import socket
from odict import odict
from turbogears import controllers, expose
from turbogears import flash, redirect, url
from turbogears import config
from operator import itemgetter
from paraxip.gateway_gui import extjs
from paraxip.gateway_gui_widgets.tabpstn  import TabPSTN
from netborderPyInfra import PyGlobalConfig as globalConfig


log = logging.getLogger("pstnconfig.controllers")

channelStatusDict = {
  'INITIALIZING':'i',
  'INITIALIZING*':'i*',
  'ALARMED':'a',
  'ALARMED*':'a*',
  'IDLE':'I',
  'IDLE*':'I*',
  'ANSWERING':'A',
  'ANSWERING*':'A*',
  'CONNECTED':'C',
  'CONNECTED*':'C*',
  'DIALING':'D',
  'DIALING*':'D*',
  'RINGING':'R',
  'RINGING*':'R*',
  'HANGING-UP':'H',
  'HANGING-UP*':'H*',
  'GOING-OUT-OF-SERVICE':'G',
  'GOING-OUT-OF-SERVICE*':'G*',
  'OUT-OF-SERVICE':'O',
  'OUT-OF-SERVICE*':'O*',
  'OUT-OF-SERVICE(FINAL)':'F',
  'OUT-OF-SERVICE(FINAL)*':'F*'
}

class StatusController(controllers.Controller):    
  def __init__(self, in_strProjectPath, in_webServiceObj):
    log.info("Initializing status controller")
    self._strServiceName = globalConfig.getParameter('netborder.oamWebServer.gwServiceName')
    
    if  self._strServiceName == None:
      log.error("Can't find name of gateway service in configuration. "
                 "Ensure that netborder.oamWebServer.gwServiceName is set "
                 "in configuration.")
      # Let's try Netborder Gateway ...
      self._strServiceName = "Netborder Gateway"
    log.debug("GW service name = %s",self._strServiceName)

    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                                         "javascript/widgets_src/status.js")

    self.webService = in_webServiceObj

    #Elements that will build our tabber
    tabs = odict()
    #Each elements of the tabs dictionnary has the format:
    # tabs['Tab Text'] = ['/autoloadUrl','tabId',"Tooltip Text"]
    tabs['Controls'] = ['','controlsTabId',"Controls of the gateway as Starting and Stopping the gateway."]
    tabs['System Status'] = ['','systemStatusTabId',"Error report and call statistics"]
    tabs['Channel Status'] = ['','channelsTabId',"See channels statuses and allows to quiesce/put in service the channels"]
    #tabs['Alarms'] = ['','alarmsTabId',""]
    
    items = [{'title': k, 'autoLoad': v[0], 'id': v[1], 'tabTip': v[2]} for k, v in tabs.items()]
    tabList = [(k.replace(" ","").lower(),v[0],v[1],v[2]) for k, v in tabs.items()]
    tabs = dict(tabs)
    
    self.statusSubTabs = TabPSTN('statusSubTabs',tabs=tabs,items=items,
                                 tabList=tabList, src_file=strSrcFile)


  def getSubTabs (self):
    return self.statusSubTabs

  @expose()
  def index (self):
    """Displays the status of the gateway using http get to localhost:7782/script/vital_sign.esp"""
    gwStarted = self.isGatewayStarted("feafea")
    return {}

  @expose()
  def restartGateway(self, _dc='None', js=''):
    self.stopGateway()
    if self.startGateway():
      return {'success':True}
    else:
      return {'success':False,'errorMsg':'Failed to start the gateway. See log file for details.'}
     
  @expose()
  def stopGatewayButton (self, _dc='None', js=''):
    return {"value":self.stopGateway()}
    
  def stopGateway(self):
    log.info("Stopping service")
    return netborderPyThread.stopService(self._strServiceName)

  @expose()
  def startGatewayButton (self, _dc='None', js=''):
    return {"value":self.startGateway()}

  def startGateway(self):
    log.info("nStarting service")
    return netborderPyThread.startService(self._strServiceName)
      
  # Use the getHostname method from the webService to test
  # wether the gateway is started or not. A socket error will
  # be raised if the gateway is not started.
  @expose()
  def pingGateway(self, _dc='None', js=''):
    try:
      self.webService.getHostname()
      return {'value':True}
    except socket.error:
      return {'value':False}

  @expose()
  def isGatewayServiceStarted(self, _dc='None', js=''):
    return {"value":self.isGWStarted()}

  def isGWStarted(self):
    return netborderPyThread.isServiceStarted(self._strServiceName) 

  @expose()
  def changeGWStartupType(self, **data):
    log.info("Changing GW Startup type: %s" % data)
    checkBoxValue = simplejson.loads(data['checkBoxValue'])
    if checkBoxValue:
      result = netborderPyThread.setServiceAutomaticStartup(self._strServiceName)
    else:
      result = netborderPyThread.setServiceManualStartup(self._strServiceName)

    return {'success':result}

  @expose()
  def isGWAutomaticStartup(self):
    return {"value":netborderPyThread.isServiceStartTypeAutomatic(self._strServiceName)}


  ##########################################################################
  # Get the Error Level and the last error from the Gateway WebService
  @expose()
  def refreshErrors(self):
    errorData = []
    errorData.append({
      'errorDesc': 'Highest Error Level reached :',
      'errorValue': self.webService.getGWAlarmState()
    })
    errorData.append({
      'errorDesc': 'Last Error Received :',
      'errorValue': self.webService.getLastAlarm()
    })
    
    return(str(extjs.FormActionLoadResponse(success=True, data=errorData)))

  ##########################################################################
  # Get the calls statistics
  @expose()
  def getStatistics(self):
    stats = []
    stats.append({
      'statName': 'Total Calls',  
      'currValue': self.webService.getNumStartedCall()
    })
    stats.append({
      'statName': 'Active Calls',  
      'currValue': self.webService.getNumActiveCall()
    })
    stats.append({
      'statName': 'Max Active Calls',  
      'currValue': self.webService.getMaxNumActiveCall()
    })
    stats.append({
      'statName': 'Average Call Duration(s)',  
      'currValue': self.webService.getAvgCallDuration()
    })
    stats.append({
      'statName': 'Min Call Duration(s)',  
      'currValue': self.webService.getMinCallDuration()
    })
    stats.append({
      'statName': 'Max Call Duration(s)',  
      'currValue': self.webService.getMaxCallDuration()
    })             
    return(str(extjs.FormActionLoadResponse(success=True, data=stats)))

  ##########################################################################
  # Resets the Error Level and the last error message
  @expose()
  def resetErrors(self, _dc='None', js=''):
    self.webService.resetAlarmState()
    return {}

  ##########################################################################
  # Put the channels out-of-service
  @expose()
  def quiesceChannels(self, _dc='None', js=''):
    self.webService.setChannelState('quiesce')
    return {}

  ##########################################################################
  # Bring the channels In Service
  @expose()
  def channelsInService(self, _dc='None', js=''):
    self.webService.setChannelState('is')
    return {}

  ##########################################################################
  # Get the channels informations
  # Received string is composed of lines with the following form:
  #  - b1(B1 - virtual)di1(B1I1)-c1 IDLE\n
  # Build a dictionnary composed of the headers of the grid and a list 
  # of all the statuses. Returned JSON string has the form:
  # data = {
  #   'headers':[
  #     {'headerName':'01','dataIndex':'01'},
  #     {'headerName':'01','dataIndex':'01'},...
  #   ],
  #   'statusList':[
  #     {'displayName':'b1di1','boardName':'B1-Virtual','interfaceName':'B1I1','01':I,'02':I,...},
  #     {'displayName':'b1di1','boardName':'B1-Virtual','interfaceName':'B1I2','01':I,'02':I,...},...
  #   ]
  # } 
  @expose()
  def getChannelStatus(self, firstCall):
    log.info("Getting Channels Status. Is First Call? %s" % firstCall)
    data = {}
    headerList = []
    statusList = []
    # Get channels status from the GW
    chanList = self.webService.getChannelStatus()
    # Replace ( and ) signs with ; to ease the data splitting.
    chanList = chanList.replace("(",";")
    chanList = chanList.replace(")",";")
    # Split the received string on the line break sign to get
    # a list of all interface-channel pair.
    chanList = chanList.splitlines()

    for channel in chanList:
      # Once splitted in a list, all lines have the form:
      #   'b1;B1 - virtual;di1;B1I1;-c1 IDLE'
      # By splitting this string on the ; sign, we get the following list:
      #   ['b1', 'B1 - virtual', 'di1', 'B1I1', '-c1 IDLE']
      splitInfo         = channel.split(";")
      displayName       = splitInfo[0] + splitInfo[2]
      boardDisplayName  = splitInfo[1]
      ifDisplayName     = splitInfo[3]
      splitInfo         = splitInfo[4].rpartition(" ")
      status            = splitInfo[2]
      status            = channelStatusDict[status]
      channel           = splitInfo[0].partition("-c")[2]
      if int(channel) < 10:
        channel = '0' + channel

      isHeaderInList = False
      for header in headerList:
        if header['headerName'] == channel:
          isHeaderInList = True
          break
      if not isHeaderInList:
        headerList.append({'headerName':channel,'dataIndex':channel})

      channelStatuses = None
      for elem in statusList:
        if elem['displayName'] == displayName:
          channelStatuses = elem
          break
      if channelStatuses:
        channelStatuses[channel] = status
      else:
        statusList.append({'displayName':displayName,channel:status,'boardName':boardDisplayName,'interfaceName':ifDisplayName})

    headerList = sorted(headerList,key=itemgetter('headerName'))
    headerList.insert(0,{'headerName':'','dataIndex':'displayName'})
    # sort is done on display name only since maximum interface number is 8.
    # if interface number grows at 10 and more, need to do a numeric sort.
    statusList = sorted(statusList,key=itemgetter('displayName'))
    #statusList.reverse()
    
    log.info("Header List: %s" % headerList)
    log.info("Status List: %s" % statusList)
    if simplejson.loads(firstCall):
      data['headers'] = headerList
      data['statusList'] = statusList
    else: 
      data = statusList
    log.info("Data: %s" % data)
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

