//-----------------------------------------------
// Add/Delete ISDN configuration methods
//-----------------------------------------------
Ext.namespace("Paraxip.config.callControl");

var addIsdnConfHandler = function(){
  addIsdnConfWindow.show();
};

function addIsdnConfig(){
  var form = addIsdnConfigForm.getForm();
  var el  = form.getEl();

  el.dom.action = "call/addIsdnConfig";
  el.dom.method = 'POST';
  var isdnConfig = Ext.util.JSON.encode(form.getValues());
  form.submit({
    params: {'isdnConfig':isdnConfig},
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      addIsdnConfWindow.hide();
      gridISDNConfiguration.store.reload();
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });
};

var deleteIsdnConfHandler = function(){
  var selectedRecords = gridISDNConfiguration.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    if ((Paraxip.gridHasDigitalBoard(gridBoards) || Paraxip.gridHasVirtualDigitalBoard(gridBoards))
          && gridISDNConfiguration.store.getCount() - selectedRecords.length < 1 ){
      Ext.Msg.alert("Can not delete last configuration!",
        "At least one configuration is necessary for the system to work properly. " +
        "Please edit current configuration or create a new one before deleting " +
        "this one");
    } else {
      var isdnConfigs = new Array();
      for (var i = 0; i < selectedRecords.length; i++) {
        isdnConfigs.push(selectedRecords[i].data);
      }
      isdnConfigs = Ext.util.JSON.encode(isdnConfigs);
      Ext.Ajax.request({
        url: 'call/deleteIsdnConfig',
        method: 'POST',
        disableCaching: false,
        params: {"isdnConfigs":isdnConfigs},
        success: function ()
        {
          Paraxip.isConfigOutOfSynch(true);
          for (var index = 0; index < selectedRecords.length; index++) {
            gridISDNConfiguration.store.remove(selectedRecords[index]);
          }
          reloadTree(true);
        },
        failure: Paraxip.failureMessage
      });
    }
  }
};

var isdnConfigChoice = new Ext.form.ComboBox({
  id              : 'isdnConfigChoice',
  fieldLabel      : "ISDN Config Type",
  store           : ['FAS','NFAS'],
  forceSelection  : true,
  triggerAction   : 'all',
  mode            : 'local',
  typeAhead       : true,
  value           : 'FAS',
  selectOnFocus   : true,
  width           : 100
});

var addIsdnConfigForm = new Ext.FormPanel({
  id          : "addIsdnConfigForm",
  region      : 'center',
  labelWidth  : 175,
  labelPad    : 10,
  bodyStyle   : 'padding:10px;',
  frame       : false,
  header      : false,
  footer      : false,
  buttonAlign : "right",
  buttons     : [{
    id: "Paraxip.config.callControl.AddIsdnConfigButton",
    text: "Add",
    listeners: {'click':addIsdnConfig}
  },{
    id: "Paraxip.config.callControl.cancelAddIsdnConfigButton",
    text: "Cancel",
    listeners: {'click':function(){addIsdnConfWindow.hide();}}
  }]
});

addIsdnConfigForm.add(isdnConfigChoice);

var addIsdnConfWindow = new Ext.Window({
  id          : "addIsdnWindow",
  layout      : 'border',
  width       : 505,
  height      : 150,
  closeAction : 'hide',
  plain       : true
});

addIsdnConfWindow.add(addIsdnConfigForm);

var gridISDNConfiguration = new Ext.grid.GridPanel({
  title       : 'ISDN Configuration Grid',
  id          : 'gridISDNConfiguration',
  autoScroll  : true,
  layout      : 'absolute',
  store       : new Ext.data.JsonStore({
    id:'storeISDNConfiguration',
    url: 'call/getGridISDNConfigurationData',
    root: 'data',
    fields: [
      {name: 'id'},
      {name: 'displayName'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns     : [
    {header: "ID", width: 35, sortable: true, dataIndex: 'id', css:'padding:5px;'},
    {header: "Name", width: 200, sortable: true, dataIndex: 'displayName', css:'padding:5px;'}
  ],
  viewConfig  : {forceFit:true}
});

gridISDNConfiguration.addButton("Add ISDN Config",addIsdnConfHandler,
                                gridISDNConfiguration);
gridISDNConfiguration.addButton("Remove ISDN Config(s)",deleteIsdnConfHandler,
                                gridISDNConfiguration);

gridISDNConfiguration.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

function isWaitFacilityDelayEnable(){
  var waitFacility;
  if( this.id == "callerNameLocationFas_select" ) {
    waitFacility = waitFacilityDelayMsFas_numfield;
  } else if( this.id == "callerNameLocationNfas_select" ) {
    waitFacility = waitFacilityDelayMsNfas_numfield;
  }

  if( this.getValue() == "IN-FACILITY-MSG" ) {
    waitFacility.enable();
  } else {
    waitFacility.disable();
    waitFacility.clearInvalid();
  }
}

//------------------------------------------------
// ISDN FAS config form
// -----------------------------------------------
function saveFasConfig(e){
  var owner = e.ownerCt;
  var form = owner.getForm();
  var el = form.getEl();

  el.dom.action = 'call/saveFasConfig';
  el.dom.method = 'POST';

  //var fasConfig = owner.getRawValues(false));
  var fasConfig = owner.getItemsValueAsJson();

  form.submit({
    params:{
      'fasConfig': fasConfig
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

//Common to Fas and Nfas
function applyToFas(e){
  var grid = e.ownerCt;
  var url;
  var params;
  var gridData;
  var records;
  var i;

  if( grid.getItemId() == "fasBChannelGrid") {
    url = "call/applyBChannelToFasConfig";
    records = grid.getStore().getRange();
    gridData = new Array();
    for (i = 0; i < records.length; i++) {
      gridData.push(records[i].data);
    }
    gridData = Ext.util.JSON.encode(gridData);
    params = {'gridData':gridData,'fasConfigID':Ext.util.JSON.encode(fasID)};
  } else if( grid.id == "nfasBChannelGrid") {
    url = "call/applyBChannelToNfasConfig";
    records = grid.getStore().getRange();
    gridData = new Array();
    for (i = 0; i < records.length; i++) {
      gridData.push(records[i].data);
    }
    gridData = Ext.util.JSON.encode(gridData);
    params = {'gridData':gridData,'nfasGroupID':Ext.util.JSON.encode(nfasID)};
  }

  Ext.Ajax.request({
    url: url,
    method: 'POST',
    disableCaching: false,
    params: params,
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      grid.ownerCt.hide();
      //reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

function addBchannel(e){
  var form = e.ownerCt;

  if( storeFasBoards.data.length < 1 ) {
    Ext.Msg.alert("No Digital Boards in config","There is no digital boards in your configuration."+
                  " Please add a board in the board section if you want to assign a board to this config.");
    return;
  }

  var firstBoardId = storeFasBoards.getAt(0).get('id');

  storeFasInterfaces.load({
    url: 'call/getInterfaces',
    disableCaching: false,
    params: {'id':firstBoardId}
  });

  if( form.id == 'fasBChannelGrid') {
    var newRecord = new DataRecord({
      'boardId':firstBoardId,
      'interfaceId':storeFasInterfaces.getAt(0).get('id'),
      'firstChannel':1,
      'lastChannel':23,
      'channelGroupID':storeFasChannelGroup.getAt(0).get('id')
    });
    fasBChannelGrid.getStore().add(newRecord);
    fasBChannelGrid.expand();
  } else if( form.id == 'nfasBChannelGrid' ) {
    var newRecord = new DataRecord({
      'boardId':firstBoardId,
      'interfaceId':storeFasInterfaces.getAt(0).get('id'),
      'nfasInterfaceID':0,
      'DChannelMode':'DISABLED',
      'firstChannel':1,
      'lastChannel':23,
      'channelGroupID':storeFasChannelGroup.getAt(0).get('id')
    });
    nfasBChannelGrid.getStore().add(newRecord);
    nfasBChannelGrid.expand();
  }

};

//Common to Fas and Nfas
function deleteBchannels(e){
  var grid = e.ownerCt;

  var selectedRecords = grid.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    for( i = 0; i < selectedRecords.length; i++ ) {
      grid.getStore().remove(selectedRecords[i]);
    }
  }

};

var storeFasBoards = new Ext.data.JsonStore({
  id      : 'storeFasBoards',
  url     : 'call/getBoards',
  root    : 'data',
  fields  : ['id','displayName','nodeId']
});

var storeFasInterfaces = new Ext.data.JsonStore({
  id      : 'storeFasInterfaces',
  url     : 'call/getInterfaces',
  root    : 'data',
  fields  : ['id','displayName']
});

var storeFasChannelGroup = new Ext.data.JsonStore({
  id      : 'storeFasChannelGroup',
  url     : 'call/getChannelGroups',
  root    : 'data',
  fields  : ['id']
});

var assignedBChannelFasWindow = new Ext.Window({
  title       : 'Assign BChannel to Fas Config',
  layout      : 'fit',
  layoutConfig: {
     autoWidth:false
  },
  width       : 600,
  height      : 500,
  closeAction : 'hide',
  resizable   : false,
  plain       : true,
  id          : "assignedBChannelFasWindow"
});

var fasBChannelGrid = new Ext.grid.EditorGridPanel({
  title       : 'BChannels assigned to config',
  id          : 'fasBChannelGrid',
  autoScroll  : true,
  width       : 550,
  height      : 400,
  clicksToEdit: 2,
  enableColumnMove : false,
  selModel    : new Ext.grid.RowSelectionModel(),
  store       : new Ext.data.JsonStore({
    id:'storeFasBChannelGrid',
    url: 'call/getFasBChannels',
    root: 'data',
    fields: [
      {name: 'boardId'},
      {name: 'interfaceId'},
      {name: 'firstChannel'},
      {name: 'lastChannel'},
      {name: 'channelGroupID'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns     : [
    {
      header: "Board ID",
      width: 110,
      sortable: true,
      dataIndex: 'boardId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasBoards,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "Interface ID",
      width: 110,
      sortable: true,
      dataIndex: 'interfaceId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasInterfaces,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "First Channel",
      width: 110,
      sortable: true,
      dataIndex: 'firstChannel',
      editor: new Ext.form.NumberField({
        allowNegative : false,
        allowBlank  : false
      })
    },
    {
      header: "Last Channel",
      width: 110,
      sortable: true,
      dataIndex: 'lastChannel',
      editor: new Ext.form.NumberField({
        allowNegative : false,
        allowBlank  : false
      })
    },
    {
      header: "Channel Group ID",
      width: 110,
      sortable: true,
      dataIndex: 'channelGroupID',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasChannelGroup,
        listClass       : 'x-combo-list-small'
      })
    }
  ],
  listeners       : {
    'afteredit':function(e){
      if( e.field == "boardId" ) {
        var colModel = e.grid.getColumnModel();
        var newColumn = e.column + 1;
        var cellEditor = colModel.getCellEditor(newColumn,e.row);
        cellEditor.field.store.load({
          url: 'call/getInterfaces',
          disableCaching: false,
          params: {'id':e.value}
        });
      }
    }
  },
  viewConfig:{forceFit:true}
});

fasBChannelGrid.store.on('load',function(store){
  store.each(function(record,index){
    var colModel = fasBChannelGrid.getColumnModel();
    colModel.getCellEditor(1,index).field.store.load({
      url: 'call/getInterfaces',
      disableCaching: false,
      params: {'id':record.data['boardId']}
    });
  });
});

fasBChannelGrid.addButton("Add BChannel",addBchannel,fasBChannelGrid);
fasBChannelGrid.addButton("Delete",deleteBchannels,fasBChannelGrid);
fasBChannelGrid.addButton("Save",applyToFas,fasBChannelGrid);
fasBChannelGrid.addButton("Cancel",function(){assignedBChannelFasWindow.hide();});

assignedBChannelFasWindow.add(fasBChannelGrid);

var fasID = '';

var frmIsdnFas = new Ext.FormPanel({
  title         : "ISDN FAS config",
  frame         : true,
  header        : false,
  id            : "frmIsdnFas",
  autoScroll    : true,
  layout        : 'accordion',
  layoutConfig  : {
    activeItem  : 0
  },
  defaults      : {
    layout: 'form',
    style: 'text-align:left;',
    labelWidth: 150,
    bodyStyle: 'padding:5px;',
    width: 500,
    autoHeight: true,
    fill: true
  },
  buttonAlign     : "right",
  buttons         : [{
    id: "Paraxip.config.callControl.assignedFasButton",
    text: 'Assigned BChannel(s)',
    listeners: {
      'click':function(){
        fasID = idFas_text.getValue();
        fasBChannelGrid.getStore().load({
          url: 'call/getFasBChannels',
          params: {'id':fasID}
        });
        storeFasBoards.load({
          url: 'call/getBoards',
          params: {'id':fasID}
        });
        storeFasChannelGroup.load({
          url: 'call/getChannelGroups',
          params: {'id':fasID}
        });
        assignedBChannelFasWindow.show();
      }
    }
  },{
    id: "Paraxip.config.callControl.saveFasButton",
    text: 'Save',
    listeners: {'click':saveFasConfig}
  },{
    id: "Paraxip.config.callControl.cancelFasButton",
    text: 'Cancel',
    listeners: {'click':callControlCancel}
  }]
});

var idFas_text = createIdText({
  id  : 'idFas_text'
});

var nameFas_text = createNameText({
  id  : 'nameFas_text'
});

var terminationFas_select = createIsdnTerminationSelection({
  id  : 'terminationFas_select'
});

var switchVariantFas_select = createIsdnSwitchVariantSelection({
  id  : 'switchVariantFas_select'
});

var BChannelNegotiationFas_select =
                            createIsdnBChannelNegotiationSelection({
  id  : 'BChannelNegotiationFas_select'
});

var initRestartProcFas_select = createIsdnInitRestartProcSelection({
  id  : 'initRestartProcFas_select'
});

var inbandProgTonesGenFas_select = createIsdnInProgTonesGenSelection({
  id  : 'inbandProgTonesGenFas_select'
});

var inbandProgTonesIndFas_select = createIsdnInProgTonesIndSelection({
  id  : 'inbandProgTonesIndFas_select'
});

var callerNameLocationFas_select = createCallerNameLocationSelection({
  id        : 'callerNameLocationFas_select',
  listeners : {
    'select':isWaitFacilityDelayEnable,
    scope:callerNameLocationFas_select
  }
});

var waitFacilityDelayMsFas_numfield = createaWaitFacilityDelayMsNumberField({
  id  : 'waitFacilityDelayMsFas_numfield'
});

var FasPanel = new Ext.Panel({
  id    : 'FasPanel',
  name  : 'fas',
  title : 'Fas Configuration',
  items : [{
    xtype: 'textfield',
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  }]
});

FasPanel.add(idFas_text);
FasPanel.add(nameFas_text);
FasPanel.add(terminationFas_select);
FasPanel.add(switchVariantFas_select);
FasPanel.add(BChannelNegotiationFas_select);
FasPanel.add(initRestartProcFas_select);
FasPanel.add(inbandProgTonesGenFas_select);
FasPanel.add(inbandProgTonesIndFas_select);
FasPanel.add(callerNameLocationFas_select);
FasPanel.add(waitFacilityDelayMsFas_numfield);
frmIsdnFas.add(FasPanel);

/* JN TODO : applyToFasConfig  */


//------------------------------------------------
// ISDN NFAS config form
// -----------------------------------------------

var storeDchannelMode = ['DISABLED','PRIMARY','BACKUP'];

var assignedBChannelNfasWindow = new Ext.Window({
  title       : 'Assign BChannel to Nfas Group',
  layout      : 'fit',
  width       : 600,
  layoutConfig: {
    autoWidth:false
  },
  height      : 500,
  closeAction : 'hide',
  resizable   : false,
  plain       : true,
  id          : "assignedBChannelNfasWindow"
});

var nfasBChannelGrid = new Ext.grid.EditorGridPanel({
  title       : 'BChannels assigned to config',
  id          : 'nfasBChannelGrid',
  autoScroll  : true,
  width       : 500,
  height      : 400,
  clicksToEdit: 2,
  enableColumnMove : false,
  selModel    : new Ext.grid.RowSelectionModel(),
  store       : new Ext.data.JsonStore({
    id:'storeNfasBChannelGrid',
    url: 'call/getNfasBChannels',
    root: 'data',
    fields: [
      {name: 'boardId'},
      {name: 'interfaceId'},
      {name: 'nfasInterfaceID'},
      {name: 'DChannelMode'},
      {name: 'firstChannel'},
      {name: 'lastChannel'},
      {name: 'channelGroupID'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns     :[
    {
      header: "Board ID",
      width: 95,
      sortable: true,
      dataIndex: 'boardId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasBoards,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "Interface ID",
      width: 95,
      sortable: true,
      dataIndex: 'interfaceId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasInterfaces,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "NFasInterface ID",
      width: 115,
      sortable: true,
      dataIndex: 'nfasInterfaceID',
      editor: new Ext.form.NumberField({
        allowNegative  : false,
        allowBlank  : false
      })
    },
    {
      header: "DChannel Mode",
      width: 110,
      sortable: true,
      dataIndex: 'DChannelMode',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        store           : storeDchannelMode,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "First Channel",
      width: 100,
      sortable: true,
      dataIndex: 'firstChannel',
      editor: new Ext.form.NumberField({
        allowNegative  : false,
        allowBlank  : false
      })
    },
    {
      header: "Last Channel",
      width: 100,
      sortable: true,
      dataIndex: 'lastChannel',
      editor: new Ext.form.NumberField({
        allowNegative  : false,
        allowBlank  : false
      })
    },
    {
      header: "Channel Group ID",
      width: 105,
      sortable: true,
      dataIndex: 'channelGroupID',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasChannelGroup,
        listClass       : 'x-combo-list-small'
      })
    }
  ],
  listeners       : {
    'afteredit':function(e){
      if( e.field == "boardId" ) {
        var colModel = e.grid.getColumnModel();
        var newColumn = e.column+1;
        var cellEditor = colModel.getCellEditor(newColumn,e.row);
        cellEditor.field.store.load({
          url: 'call/getInterfaces',
          disableCaching: false,
          params: {'id':e.value}
        });
      }
    }
  },
  viewConfig:{forceFit:true}
});

nfasBChannelGrid.store.on('load',function(store){
  store.each(function(record,index){
    var colModel = nfasBChannelGrid.getColumnModel();
    colModel.getCellEditor(1,index).field.store.load({
      url: 'call/getInterfaces',
      disableCaching: false,
      params: {'id':record.data['boardId']}
    });
  });
});

nfasBChannelGrid.addButton("Add BChannel",addBchannel,nfasBChannelGrid);
nfasBChannelGrid.addButton("Delete",deleteBchannels,nfasBChannelGrid);
nfasBChannelGrid.addButton("Save",applyToFas,nfasBChannelGrid);
nfasBChannelGrid.addButton("Cancel",function(){assignedBChannelNfasWindow.hide();});

assignedBChannelNfasWindow.add(nfasBChannelGrid);

var nfasID = '';

function saveNfasConfig(e){
  var owner = e.ownerCt;
  var form = owner.getForm();
  var el = form.getEl();

  el.dom.action = 'call/saveNfasConfig';
  el.dom.method = 'POST';

  var nfasGroup = owner.getItemsValueAsJson();

  form.submit({
    params:{
      'nfasGroup': nfasGroup
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

var frmIsdnNfas = new Ext.FormPanel({
  title         : "ISDN NFAS config",
  frame         : true,
  header        : false,
  id            : "frmIsdnNfas",
  autoScroll    : true,
  layout        : 'accordion',
  layoutConfig  : {
    activeItem  : 0
  },
  defaults      : {
    layout: 'form',
    style: 'text-align:left;',
    labelWidth: 150,
    bodyStyle: 'padding:5px;',
    width: 500,
    autoHeight: true,
    fill: true
  },
  buttonAlign   : "right",
  buttons       : [{
    id: "Paraxip.config.callControl.assignedNfasButton",
    text: 'Assigned BChannel(s)',
    listeners: {
      'click':function(){
        nfasID = idNfas_text.getValue();
        nfasBChannelGrid.getStore().load({
          url: 'call/getNfasBChannels',
          params: {'id':nfasID}
        });
        storeFasBoards.load({
          url: 'call/getBoards',
          params: {'id':nfasID}
        });
        storeFasChannelGroup.load({
          url: 'call/getChannelGroups',
          params: {'id':nfasID}
        });
        assignedBChannelNfasWindow.show();
      }
    }
  },{
    id: "Paraxip.config.callControl.saveNfasButton",
    text: 'Save',
    listeners: {'click':saveNfasConfig}
  },{
    id: "Paraxip.config.callControl.cancelNfasButton",
    text: 'Cancel',
    listeners: {'click':callControlCancel}
  }]
});

var NfasPanel = new Ext.Panel({
  id    : 'NfasPanel',
  name  : 'nfas',
  title : 'Nfas Configuration',
  items : [{
    xtype: 'textfield',
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  }]
});

var idNfas_text = createIdText({
  id  : 'idNfas_text'
});

var nameNfas_text = createNameText({
  id  : 'nameNfas_text'
});

var terminationNfas_select = createIsdnTerminationSelection({
  id  : 'terminationNfas_select'
});

var switchVariantNfas_select = createIsdnSwitchVariantSelection({
  id  : 'switchVariantNfas_select'
});

var BChannelNegotiationNfas_select =
                            createIsdnBChannelNegotiationSelection({
  id  : 'BChannelNegotiationNfas_select'
});

var initRestartProcNfas_select = createIsdnInitRestartProcSelection({
  id  : 'initRestartProcNfas_select'
});

var inbandProgTonesGenNfas_select = createIsdnInProgTonesGenSelection({
  id  : 'inbandProgTonesGenNfas_select'
});

var inbandProgTonesIndNfas_select = createIsdnInProgTonesIndSelection({
  id  : 'inbandProgTonesIndNfas_select'
});

var callerNameLocationNfas_select = createCallerNameLocationSelection({
  id  : 'callerNameLocationNfas_select',
  listeners : {
    'select':isWaitFacilityDelayEnable,
    scope:callerNameLocationNfas_select
  }
});

var waitFacilityDelayMsNfas_numfield = createaWaitFacilityDelayMsNumberField({
  id  : 'waitFacilityDelayMsNfas_numfield'
});


NfasPanel.add(idNfas_text);
NfasPanel.add(nameNfas_text);
NfasPanel.add(terminationNfas_select);
NfasPanel.add(switchVariantNfas_select);
NfasPanel.add(BChannelNegotiationNfas_select);
NfasPanel.add(initRestartProcNfas_select);
NfasPanel.add(inbandProgTonesGenNfas_select);
NfasPanel.add(inbandProgTonesIndNfas_select);
NfasPanel.add(callerNameLocationNfas_select);
NfasPanel.add(waitFacilityDelayMsNfas_numfield);
frmIsdnNfas.add(NfasPanel);

