Ext.namespace("Paraxip.status.channels");

// This map contains all the colors associated with the channel statuses.
Paraxip.status.channels.StatusColorMap = new Paraxip.Map();
Paraxip.status.channels.StatusColorMap.put('i','#ffff00');
Paraxip.status.channels.StatusColorMap.put('i*','#ffff00');
Paraxip.status.channels.StatusColorMap.put('a','#ff6347');
Paraxip.status.channels.StatusColorMap.put('a*','#ff6347');
Paraxip.status.channels.StatusColorMap.put('I','#87cefa');
Paraxip.status.channels.StatusColorMap.put('I*','#87cefa');
Paraxip.status.channels.StatusColorMap.put('A','#98fb98');
Paraxip.status.channels.StatusColorMap.put('A*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('D','#98fb98');
Paraxip.status.channels.StatusColorMap.put('D*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('R','#98fb98');
Paraxip.status.channels.StatusColorMap.put('R*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('C','#98fb98');
Paraxip.status.channels.StatusColorMap.put('C*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('H','#98fb98');
Paraxip.status.channels.StatusColorMap.put('H*','#98fb98');
Paraxip.status.channels.StatusColorMap.put('G','#ffff00');
Paraxip.status.channels.StatusColorMap.put('G*','#ffff00');
Paraxip.status.channels.StatusColorMap.put('O','#ffff00');
Paraxip.status.channels.StatusColorMap.put('O*','#ffff00');

Paraxip.status.channels.updateChannelGrid = function(){
  var gridView = Paraxip.status.channels.channelGrid.getView();
  var store    = Paraxip.status.channels.channelGrid.getStore();
  var colModel = Paraxip.status.channels.channelGrid.getColumnModel();
  var colCount = colModel.getColumnCount();
  var rowCount = store.getTotalCount();

  for( var i = 0; i < rowCount; i++ ) {
    for( var j = 0; j < colCount; j++ ) {
      var rec = store.getAt(i); // SelectionModel might not give you one single row!
      var colName = colModel.getDataIndex(j); // Columns might move!
      var val = rec.get(colName);
      var cell = gridView.getCell(i,j);
      var color = Paraxip.status.channels.StatusColorMap.get(val);
      cell.setAttribute('bgColor',color);    
    }
  }

  Paraxip.status.channels.resizeGrid(colCount,rowCount);
}

Paraxip.status.channels.quiesceChannels = function(){
  Ext.Ajax.request({
    url: 'status/quiesceChannels',
    method: 'POST',
    disableCaching: true,
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function(){
      Paraxip.status.channels.updateChannelStatus();          
    },
    failure: Paraxip.failureMessage
  });
}

Paraxip.status.channels.channelsInService = function(){
  Ext.Ajax.request({
    url: 'status/channelsInService',
    method: 'POST',
    disableCaching: true,
    params: {js:Ext.util.JSON.encode('dummy param')},
    success: function(){
      Paraxip.status.channels.updateChannelStatus();          
    },
    failure: Paraxip.failureMessage
  });
}

Paraxip.status.channels.updateChannelStatus = function(){
  Paraxip.status.channels.channelGrid.getStore().reload();
}

Paraxip.status.channels.resizeGrid = function(numCol,numRow){
  if( numCol <= 6) {
    var width = 350;
  } else {
    var width = 45 + (numCol-1) * 25;
  }
  var height = 103 + 20*(numRow);

  var x = (970-width) / 2;
  var y = (482-(height)) / 2;

  Paraxip.status.channels.channelGrid.setSize(width, height);
  Paraxip.status.channels.channelGrid.setPosition(x,y);
}

Paraxip.status.channels.reconfigureGrid = function(data){
  colArray = new Array();
  fieldsArray = new Array();
  var colHeaders = data.headers;
  var statusData = data.statusList;

  //TODO - keep trace of the board and interface display name for further display
  for( var i = 0; i < colHeaders.length; i++ ) {
    if ( i == 0) {
      var width = 40;
    } else {
      var width = 25;
    }
    colArray.push({header:colHeaders[i].headerName, width:width, dataIndex:colHeaders[i].dataIndex, css: 'text-align:center;'});
    fieldsArray.push({name:colHeaders[i].dataIndex});
  }
  Paraxip.status.channels.resizeGrid(colArray.length,statusData.length);

  var store = new Ext.data.JsonStore({
    id: 'channelGridStore',
    fields: fieldsArray,
    url: 'status/getChannelStatus',
    baseParams : {
      firstCall:Ext.util.JSON.encode(false)
    },
    root: 'data',
    listeners: {
      'load':Paraxip.status.channels.updateChannelGrid
    }
  });

  var colModel = new Ext.grid.ColumnModel(colArray);
  Paraxip.status.channels.channelGrid.reconfigure(store,colModel);
  Paraxip.status.channels.channelGrid.getStore().load({url: 'status/getChannelStatus'});
  Paraxip.status.channels.channelGrid.getStore().startAutoRefresh(Paraxip.status.controls.autoRefreshTime.getRawValue());
  Paraxip.status.channels.channelGrid.getStore().on('load',Paraxip.status.channels.updateChannelGrid);
}

Paraxip.status.channels.legendWindow = new Ext.Window({
  id          : 'Paraxip.status.channels.legendWindow',
  title       : 'Channel Status legend',
  closeAction : 'hide',
  plain       : true,
  hidden      : true,
  width       : 500,
  height      : 400,
  items       : [{
    xtype: 'label',
    html: "<html><br><font size='2'>I: Idle</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>i: initializing</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>a: alarmed</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>A: Answering</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>C: Connected</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>D: Dialing</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>R: Ringing</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>H: Hanging-up</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>G: Going-out-of-service</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>O: Out-of-service</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>F: out-of-service(Final)</font><br></html>"
  },{
    xtype: 'label',
    html: "<html><br><font size='2'>*: Star appended to a status means the system is in transition to another status</font><br></html>"
  }]
});

Paraxip.status.channels.channelGrid = new Ext.grid.GridPanel({
  id: 'Paraxip.status.channels.channelGrid',
  title: 'PSTN Line Status',
  cls: 'channelGrid',
  autoScroll: true,
  stripeRows: true,
  enableHdMenu: false,
  enableColumnMove: false,
  hideHeaders: false,
  border: true,
  height: 360,
  style: 'text-align:left;white-space: normal;',
  store: new Ext.data.JsonStore({
    id: 'channelGridStore',
    fields: [
      {name: 'displayName'},
    ]
  }),
  columns: [],
  viewConfig :{forceFit:true,scrollOffset: 0, autoFill: true},
  buttons:[{
    id: 'Paraxip.status.channels.refreshButton',
    text: 'Refresh',
    listeners: {
      'click':function(){
        Paraxip.status.channels.channelGrid.getStore().reload();
      }
    }
  },{
    id: 'Paraxip.status.channels.showLegendButton',
    text: 'Show Legend',
    listeners: {
      'click':function(){
        Paraxip.status.channels.legendWindow.show();
      }
    }
  },{
    id: 'Paraxip.status.channels.quiesceButton',
    text: 'Quiesce',
    listeners: {
      'click':Paraxip.status.channels.quiesceChannels
    }
  },{
    id: 'Paraxip.status.channels.inServiceButton',
    text: 'In Service',
    listeners: {
      'click':Paraxip.status.channels.channelsInService
    }
  }]
});
  
Paraxip.status.channels.channelPanel = new Ext.Panel({
  id: 'Paraxip.status.channels.channelPanel',
  layout: 'absolute',
  width: 971,
  height: 486,
  bodyStyle: 'background-color: #DFE8F6;',
  items: [Paraxip.status.channels.channelGrid]
});

Paraxip.status.channels.channelGrid.on('render',function(){
  Ext.Ajax.request({
    url: 'status/getChannelStatus',
    method: 'GET',
    params: {
      firstCall:Ext.util.JSON.encode(true)
    },
    success: function(form, action){
      var response = Ext.util.JSON.decode(form.responseText);
      Paraxip.status.channels.reconfigureGrid(response.data);
    }
  });
});

